<?php

namespace App\Service;

use App\Dto\BarChart\BarChartInput;
use App\Dto\BarChart\BarChartOutput;
use Doctrine\ORM\EntityManagerInterface;
use App\Entity\Sale;

class BarChartService
{
    private EntityManagerInterface $entityManager;

    public function __construct(EntityManagerInterface $entityManager)
    {

        $this->entityManager = $entityManager;
        $emConfig = $this->entityManager->getConfiguration();
        $emConfig->addCustomDatetimeFunction('YEAR', 'DoctrineExtensions\Query\Postgresql\Year');
        $emConfig->addCustomDatetimeFunction('MONTH', 'DoctrineExtensions\Query\Postgresql\Month');
        $emConfig->addCustomDatetimeFunction('DATE_FORMAT', 'DoctrineExtensions\Query\Postgresql\DateFormat');

    }

    public function getBarChartData(BarChartInput $input): array
    {
        $startDate = new \DateTime($input->start);
        $endDate = new \DateTime($input->end);

        $queryBuilder = $this->entityManager->createQueryBuilder();

        switch ($input->granularity) {
            case 'day':
                $groupByExpression = 's.date';
                $groupByAlias = 'date';
                $dateFormat = 'Y-m-d';
                break;
            case 'month':
                $groupByExpression = 'MONTH(s.date)';
                $groupByAlias = 'month';
                $dateFormat = 'Y-m';
                break;
            case 'year':
                $groupByExpression = 'YEAR(s.date)';
                $groupByAlias = 'year';
                $dateFormat = 'Y';
                break;
            default:
                throw new \InvalidArgumentException('Invalid granularity');
        }

        $result = $queryBuilder
            ->select("{$groupByExpression} as {$groupByAlias}", 'COUNT(s.id) as occurrences')
            ->from(Sale::class, 's')
            ->where('s.date BETWEEN :start AND :end')
            ->setParameter('start', $startDate->format('Y-m-d'))
            ->setParameter('end', $endDate->format('Y-m-d'))
            ->groupBy("{$groupByAlias}")
            ->orderBy("{$groupByAlias}")
            ->getQuery()
            ->getResult();

        $output = [];
        foreach ($result as $row) {
            $dateString = $row[$groupByAlias] instanceof \DateTimeInterface ? $row[$groupByAlias]->format($dateFormat) : $row[$groupByAlias];
            $output[] = new BarChartOutput($dateString, (int)$row['occurrences']);
        }

        return $output;
    }

}
